/*

 AD9952 DDS chip driver

 OH2NLT Juha Niinikoski 16.03.2008

 First test setup

 20MHz Xtal 
 Mult = 18 ==> Ref CLK = 360MHz
 Mult = 20 ==> Ref CLK = 400MHz

 Output = 2 * TRX2 DDS output freq or 4 * display freq

Some test board power concumption measurements

Ref CLK = 360MHz

Out 10 MHz
Board 117mA

Out 160MHz
Board 118mA
DVDD 40mA
AVDD 24mA

Ref CLK = 400MHz

Out 200MHz
Board 124mA
DVDD 43mA
AVDD 26mA

*/


#include "juma-trx2.h"						// JUMA-TRX2 hardware specific definitions


// AD9952 DDS / test I/O
//#define AD9952_SDI		_LATG9			// AD9952 data I/O
//#define AD9952_CLK		_LATG12			// AD9952 data write clock
//#define AD9952_CS			_LATG13			// AD9952 nCS
//#define AD9952_IOUPD		_LATG8			// AD9952 I/O update
//#define AD9952_RESET		_LATG14			// AD9952 reset
//#define AD9952_SB_SEL		_LATG15			// sideband select	


// some AD9952 register definitions

// Instruction byte / register adresses
#define RW			(1<<7)				// R/W bit, 0 = write
#define CFR1		0x00				// Control Function Register 1
#define CFR2		0x01				// Control Function Register 2
#define ASF			0x02				// Amplitude Scale Factor
#define ARR			0x03				// Amplitude Ramp Rate
#define FTW0		0x04				// Frequency Tuning Word 0
#define POW 		0x05				// Phase Offset Word


// CFR1 bit definitions, 24-bit long
#define SYN_CLK_DIS			(1<<1)		// SYN_CLK Out Disable
#define	LSB_FIRST			(1<<8)		// LSB first data transfer

// CFR2 bit definitions, 24-bit long
#define CHARGE_PUMP_CURRENT	(3<<0)		// Charge Pump Current, 0=75uA, 25uA/bit, 1=100uA 
#define VCO_RANGE			(1<<2)		// VCO Range, 0=100 to 250MHz, 1=250 to 400 MHz
#define REFCLK_MULT			(0x1F<<3)	// REFCLK Multiplier, range 4 to 20, 0 = bypass
#define XOUT_ACTIVE			(1<<9)		// XTAL osc out pin active, 1 = active

// ASF bit definitions, 16-bit long

// ARR bit definitions, 8-bit long

// FTW0 , 32-bit long

// POW bit definitions, 16-bit long


// generate I/O update pulse
void ad9952_ioupdate(void)
	{
	AD9952_IOUPD = 1;
	__asm__ volatile ("nop");
	__asm__ volatile ("nop");
	__asm__ volatile ("nop");
	__asm__ volatile ("nop");
	AD9952_IOUPD = 0;
	}


// map ad9952 registers to unsigned long 
static	union									// ad9952 frequency & config data, index 0 = LSB
	{
	unsigned long dds_freq;
	unsigned char ad9952[4];
	} dds;



/* ad9952 data transfer */

void ad9952_send_one( unsigned char dta )		// send one byte MSB first
	{
	char x;
	for(x=0; x<8; x++)			// do all bits
		{
		if(dta & 0x80)
			AD9952_SDI = 1;
		else
			AD9952_SDI = 0;

		AD9952_CLK = 1;						// generate clock pulse
		dta = dta << 1;						// move next bit
		AD9952_CLK = 0;	
		}
	}


void ad9952_load(unsigned long set_val)		// load ad9952 from buffer			
	{
	int x;
	dds.dds_freq = set_val;

	AD9952_CS = 0;							// select AD9952 chip
	ad9952_send_one(FTW0);					// send register address

	for(x=3; x>-1; x--)
		{
		ad9952_send_one(dds.ad9952[x]);		// send 40 bits
		}

	AD9952_CS = 1;
	ad9952_ioupdate();						// do I/O update
	}


/* Init ad9952 interface */

void ad9952_init(void)
	{
// set I/O signals
	AD9952_RESET = 1;						// reset ad9952
	AD9952_CS = 1;							// set I/O signals
	AD9952_CLK = 0;
	AD9952_SDI = 0;
	AD9952_RESET = 0;						// ready to run

// write control registers	
	AD9952_CS = 0;							// select AD9952 chip

// CFR2
	ad9952_send_one(CFR2);					// send register address
	ad9952_send_one(0x00);					// [23:16]
	ad9952_send_one(0x02);					// [15:8], XOUT active
//	ad9952_send_one((9<<3));				// [7:0], Mult = 9, VCO range = 250MHz
//	ad9952_send_one((18<<3) | VCO_RANGE);	// [7:0], Mult = 18, VCO range = 400MHz, 18*20MHz = 360MHz RefCLK
	ad9952_send_one((20<<3) | VCO_RANGE);	// [7:0], Mult = 20, VCO range = 400MHz, 20*20MHz = 400MHz RefCLK

	AD9952_CS = 1;
	ad9952_ioupdate();						// do I/O update
	}

