//
// JUMA-TRX2 serial I/O test command set
//
// Juha Niinikoski, OH2NLT
//
// Serial test commands moved to own module 03.01.2007
//

// PA RF attenuator test 23.04.2007
// EEPROM Dump limits increaced 08.03.2008

#include "juma-trx2.h"				// board definitions
#include "DataEEPROM.h"				// EEPROM I/O definitions, interface to assembler module
#include "lcd-trx2.h"				// LCD specific definitions

// external references
// external functions

extern void putch(char);						// UART1 I/O
extern int kbhit(void);
extern unsigned char getch(void);
extern unsigned char getche(void);
extern unsigned long getlong(void);
extern void bin2ascii(unsigned long);
extern void putlong( unsigned long);
extern void rs232_debug(void);

extern void InitUSART2(void);					// UART2 / secondary uart
extern void SetUSART2baud(int);	
extern void putch2(char);
extern int kbhit2(void);
extern unsigned char getch2(void);
extern unsigned char getche2(void);
extern void putst2(char *); 

extern void draw_s_meter(int);					// LCD functions
extern void draw_bar(int, int);
extern void lcd_putst(register const char *);
extern void set_chgen(void);
extern void initlcd(void);
extern void clear_lcd(void);
extern void set_cur_lcd(unsigned char);
extern void disp_freq(long);
extern void disp_rit(int);
extern void disp_meter(int, int);

extern void init_adc12(void);
extern int convert_adc12(unsigned int adchs);	// ADC

extern void init_spi1(void);					// SPI, main board 74HC595 I/O latch
extern void exchg_data_spi1(unsigned int);

extern void start_scaf_clock(void);				// start / stop scaf clocking
extern void stop_scaf_clock(void);

extern void ms_delay( unsigned int);			// general delay


// external data

extern const char ver[];			// system version
extern unsigned char lcdpbuff[40];	// LCD print buffer, longer than the display for for prints

extern int batt;					// Powersupply voltage scaled value, measured only when displayed
extern int service_mode;			// service mode selected flag

extern unsigned int long_push;		// long push timer for buttons

extern int s_meter;
extern int cw_pot;

extern unsigned int rf_attn;		// PA RF attenuator
extern int	rf_filt_idx;			// RF filter & attenuator switch table index
extern unsigned int hc595;			// main & PA board SPI bus bits

extern int vfo_idx;					// VFO index, A=0, B=1, if(Split) then  RX=0, TX=1

extern int dds_test;				// DDS test, tuning word +/-
extern int dds_test_idx;			// select tuning word +/- step
extern const long f_test_steps[];
extern int freq_disp_test;			// frequency display, 0 = normal, others = HEX DDS set value 

extern int cw_active;				// status flag for keyer module
extern int tune_on;					// tune state indicator
extern int PTT_IN;					// atomic copy of PTT_IN_IO input / TX on, 0 = active
extern int mute;					// mute flag

// EEPROM structures

#include "trx2_eeprom.h"			// get EEPROM structure definitions

extern union
	{
	struct defval defval;
	unsigned int storage[sizeof(struct defval)/2];
	}eeprom;

// system calibration values

extern union
	{
	struct calval calval;
	unsigned int ee[sizeof(struct calval)/2];
	}cal;


void dump_eeprom(void)
	{
	int x;
	unsigned int y, w;

	printf("Dump EEPROM contents\n\r");	
	for(x=0; x<512; x=x+2)								// byte address for EEPROM, but data is stored in words
		{
		y = ReadEE(EEPAGE, (x+EEDEF), &w, WORD);		// EEPROM Address 8 high bits, address + physical EEPROM start 16 low bits
		if((x%16) == 0)
			printf("\n\r%5.4X", x);

			printf(" %4X", w);			
		}
	printf("\n\rLast success code = %X ", y);
	}

//_______________________________________________________

#define FUN 	0x8000			// treated as function code
#define LET		0x4000			// letter. Capital if type == 0x01

const unsigned int DoCoMo_scan_codes[256] = {					// table start at 0x00

0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,		// 0x00..0x07
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xFF, 0x00,		// 0x08..0x0F

0x00, 0xFF, 0xFF, 0xFF, 0xFF, 0x00, ('A'+LET), ('B'+LET),									// 0x10..0x17
('C'+LET), ('D'+LET), ('E'+LET), ('F'+LET), ('G'+LET), ('H'+LET), ('I'+LET), ('J'+LET),		// 0x18..0x1F

0x00, 0x00, 0x00, 0x00, 0x00, 0x00, ('K'+LET), ('L'+LET),									// 0x20..0x27
('M'+LET), ('N'+LET), ('O'+LET), ('P'+LET), ('Q'+LET), ('R'+LET), ('S'+LET), ('T'+LET),		// 0x28..0x2F, dublicates 0x2D = Red phone, 0x2E = Green phone

0x00, ' ', 0xFF, 0xFF, 0x00, 0x00, ('U'+LET), ('V'+LET),									// 0x30..0x37, dublicates 0x36 = info button
('W'+LET), ('X'+LET), ('Y'+LET), ('Z'+LET), 0x00, 0x00, 0x00, 0x00,							// 0x38..0x3F

0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,		// 0x40..0x47
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,		// 0x48..0x4F

0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,		// 0x50..0x57
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,		// 0x58..0x5F

0x00, 0x00, 0xFF, 0x08, 0xFF, 0x00, 0x00, 0x00,		// 0x60..0x67
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,		// 0x68..0x6F

0x00, 0x00, 0x00, 0x00, 0x00, 0x00, '@', '.',		// 0x70..0x77
'/', ',', '-', 0xFF, ':', 0x27, 0x00, '&',			// 0x78..0x7F

0x00, 0x00, '(', 0x00, ')', 0x00, '"', '$',			// 0x80..0x87
'%', '+', ';', '<', '=', '>', 0XFF, '[',			// 0x88..0x8F

0x00, 0x00, 0x00, 0x00, 0x00, 0x00, ']', '^',		// 0x90..0x97
0x00, '|', 0x00, 0xFF, 0xFF, '~', 0x00, 0x60,		// 0x98..0x9F

0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,		// 0xA0..0xA7
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,		// 0xA8..0xAF

0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,		// 0xB0..0xB7
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,		// 0xB8..0xBF

0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,		// 0xC0..0xC7
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,		// 0xC8..0xCF

0x00, 0x60, 0xFF, 0xFF, '!', '?', 0x00, 0x00,		// 0xD0..0xD7
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,		// 0xD8..0xDF

0x00, '1', '2', '3', '4', '5', 0x00, 0x00,			// 0xE0..0xE7
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,		// 0xE8..0xEF

0x00, '6', '7', '8', '9', '0', '#', '*',				// 0xF0..0xF7
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00		// 0xF8..0xFF
};


// Data decoder for strange
// NTT DoCoMo i BOARD
 
// KB transmits 8 byte long packets at 600bd
// packets are 0xF2, 0x02, 0x01, 0x00, 0x00, type, scnHi, scnLO
// Type: 0x01, 0x02, 0x03
// 0x02 = right Red light on = "normal" operation
// 0x01 = both Red lights on = "normal" operation + Shift
// 0x03 = Command buttons
// if no Red lights are on mode = Japanese ???, some sort of overlay system
//

void DoCoMo(unsigned char key)
	{
	static int ctr;
	static unsigned char type;
	static unsigned char scan_code;

	unsigned int temp;
	unsigned char letter;

	if(key == 0)
		{
		ctr = 0;										// characters [3] and [4] == 0, good way to sync the message
		clear_lcd();
		return;											// Init state machine & leave the function
		}

	switch(ctr)											// start processing from [5]
		{
		case 0:											// save message type [5]
		type = key;
		break;


		case 1:											// save scan code high part [6]
		scan_code = key << 4;
		break;

		case 2:											// save low part, RX ready for decoding

		scan_code = scan_code | (key & 0x0F);			// combine scan code

		if(type == 0x03)								// before look up table processing filter some (dublicate) command button codes
			{
			if(scan_code == 0x2D)						// Red telephone
				lcd_putst("Red ");

			if(scan_code == 0x2E)						// Green telephone
				lcd_putst("Green ");

			if(scan_code == 0x36)						// i Info button
				lcd_putst("info ");
			}

		temp = DoCoMo_scan_codes[scan_code];			// do look up

		if(temp & LET)									// letters flag set
			{
			if(type == 0x02)							// if shift  is off, convert lower case
				letter = (temp &0x00FF) + ('a'-'A');
			else
				letter = temp &0x00FF;					// else leave capital
			}
		else
			{
			letter = temp &0x00FF;						// other printaple characters
			}


		lcd_putchhex(type);
		lcd_putch(' ');
		lcd_putchhex(scan_code);
		lcd_putch(' ');
		lcd_putch(letter);
		break;


		default:
		lcd_putch('?');
		}

	ctr++;

	}

//______________________________________________________________________

void serial_test(void)
	{
	int x, z, temp;
	unsigned int y, w;
	unsigned char c,ctr;

		if(kbhit())
			{
			c = getch();
			switch(c)
				{

// Info commands
				case 'I':												// system info
					printf("\n\rJUMA-TRX2 copyright OH7SV & OH2NLT\n\r");
					printf("\n\rSoftware version ");
					printf(ver);
					printf(", copyright Juha Niinikoski OH2NLT\n\r");
					break;

// Test commands

// ADC & PWM DAC test
				case 'A':												// Measure all ADC channels
				for(x=9; x<16; x++)
						{
						printf("\n\rADC %i = %i", x, convert_adc12(x));
						}
					break;

// EEPROM
				case 'E':												// dump EEPROM
					dump_eeprom();
					break;

				case 'C':												// clear factory default reset counter
					printf("EEPROM reset counter cleared\n\r");
					temp = 0;
					EraseEE(EEPAGE, EE_FD_LOC, WORD);
					WriteEE(&temp, EEPAGE, EE_FD_LOC, WORD);
					dump_eeprom();
					break;

// LCD tests
				case 'W':												// test LCD chgen
					printf("LCD write\n\r");
					while(c != 0x1B)
						{
						while(kbhit() == 0);
						c = getch();
					printf("char = %x\n\r", c);
						lcd_putch(c);
						}
					break;

				case 'B':												// bar graph
					printf("LCD bar graph & character test\n\r");
					c = 0;
					clear_lcd();
					for(;;)
					{
					for(x=0; x<96; x++)
						{
						set_cur_lcd(LINE1);
						draw_s_meter(x);
						ms_delay(50);
						}

					set_cur_lcd(LINE2);
					sprintf(lcdpbuff, "%c = 0x%X  ",c, c);		// display LCD characters
					lcd_putst(lcdpbuff);
					c++;

					if(kbhit())
						break;
					}
					break;

// Audio tests

				case 's':												// stop scaf, mute
					printf("\n\rMute");
					mute = 1;											// set flag foe squelch code
					break;

				case 'S':												// start scaf, audio on
					printf("\n\rAudio on");
					mute = 0;
					break;

				case 'p':												// squelch variables
					printf("\n\rS_meter = %1,POT = %i\n\r",s_meter, cw_pot );
					break;


// Temporary hardware debug commands

				case 'd':						// test, debug print some tuning variables
					printf("\n\r eeprom.defval.vfo_sel = %i", eeprom.defval.vfo_sel);
					printf("\n\r vfo_idx = %i", vfo_idx);
					printf("\n\r eeprom.defval.mode[vfo_idx] = %i", eeprom.defval.mode[vfo_idx]);
					printf("\n\r cw_active = %i", cw_active);
					printf("\n\r tune_on = %i", tune_on);
					printf("\n\r PTT_IN = %i", PTT_IN);
					break;

				case 'D':						// Debug RS232 input, cross copy
					clear_lcd();
					c = 0;
					ctr = 0;
	

//					printf("\n\r U1STA = %X,  U2STA= %X", U1STA, U2STA);
//					putst2("\n\rJUMA-TRX2 UART2 test\n\r");

//					while(c != 0x1B)			// exit with ESC
					while(1)			// test, never exit
						{

					if(kbhit())
						{
						c = getch();
						putch2(c);
						set_cur_lcd(LINE1);		
						lcd_putch(c);
						}

					if(kbhit2())
						{
						c = getch2();
						putch2(c);
//						putch(c);

						if(ctr%8 == 0)
							printf("\n\rKB > ");

						printf("%X ", c);

//						set_cur_lcd(LINE2);


						DoCoMo(c);		// test decode
#if 0
						lcd_putch(' ');		// test
						if(ctr == 5)
							set_cur_lcd(LINE2);
						lcd_putchhex(c);
#endif

						ctr++;
						}

						if(SW5 == 0)
							{
							clear_lcd();
							ctr = 0;
							}
						}
//					rs232_debug();
					break;

				case 'o':						// test, print ref osc calibration value
					printf("\n\rRef Osc = %li MHz\n\r", cal.calval.ref_osc);
					break;


				case 'm':						// test, print ms timing
					printf("\n\rms counter = %i\n\r", get_ms_counter());
					break;

				case 't':						// test write SPI bus
					printf("\n\rSPI Tx");
					exchg_data_spi1(0x5501);
					break;

				case 'c':						// continous test write SPI bus
					printf("\n\rSPI Tx cont");
					for(;;)
						{
						y++;
						exchg_data_spi1(y);
						ms_delay(10);
						if(kbhit())
							break;
						}
					break;

				case '+':						// test
					cal.calval.rf_atn_bits[rf_filt_idx]++;
					cal.calval.rf_atn_bits[rf_filt_idx] = cal.calval.rf_atn_bits[rf_filt_idx] & 0x03;
					printf("\n\rPA Attenuator = 0x%X", cal.calval.rf_atn_bits[rf_filt_idx] );
					break;

				case '-':						// test
					cal.calval.rf_atn_bits[rf_filt_idx]--;
					cal.calval.rf_atn_bits[rf_filt_idx] = cal.calval.rf_atn_bits[rf_filt_idx] & 0x03;
					printf("\n\rPA Attenuator = 0x%X", cal.calval.rf_atn_bits[rf_filt_idx] );	
					break;

				case 'f':						// test, print current SPI bus control bits
					printf("\n\rRF Attn select idx = %i, HC595 bits = 0x%X", rf_filt_idx, hc595);
					break;


				case '0':						// toggle DDS test mode
					if(dds_test == 0)
						{
						printf("\n\rDDS tune test ON");
						dds_test = 1;
						}
					else
						{
						printf("\n\rDDS tune test Off");
						dds_test = 0;
						}
					break;

				case '9':						// toggle DDS freq display mode
					if(freq_disp_test == 0)
						{
						printf("\n\rDDS tuning word display");
						freq_disp_test = 1;
						}
					else
						{
						printf("\n\rNormal Frequency display");
						freq_disp_test = 0;
						}
					break;

				case '1': case '2': case '3': case '4': case '5': case '6':
					dds_test_idx = (c & 0x0F) - 1;
					printf("\n\rDDS tuning step # %i = %li", dds_test_idx, f_test_steps[dds_test_idx]);
					break;


				case 'Z':						// Trap / Error test
					printf("\n\rDiv by Zero");
					x = 0;
					temp = temp / x;			// generate error ==> trap
					break;

				default:
					printf("Key is: %d\n\r", c);
					printf("Try: Valid commands\n\r");

				}	// case
			}		// if
	}
